@@ -29,6 +29,7 @@
 #include "mailcheck.h"
 
 #include "egg-screen-help.h"
+#include "egg-screen-exec.h"
 
 typedef enum {
 	MAILBOX_LOCAL,
@@ -122,6 +123,8 @@ struct _MailCheck {
 	/* the about box */
 	GtkWidget *about;
 
+	GtkWidget *password_dialog;
+
 	gboolean anim_changed;
 
 	char *mailcheck_text_only;
@@ -143,8 +146,12 @@ struct _MailCheck {
 
 	int type; /*mailcheck = 0; mailbox = 1 */
 	
+	off_t oldsize;
+	
 	int size;
 
+	gulong applet_realized_signal;
+
 	/* see remote-helper.h */
 	gpointer remote_handle;
 };
@@ -157,6 +164,8 @@ static void applet_load_prefs(MailCheck *mc);
 static void set_atk_name_description (GtkWidget *widget, const gchar *name,
 					const gchar *description);
 static void set_atk_relation (GtkWidget *label, GtkWidget *entry, AtkRelationType);
+static void got_remote_answer (int mails, gpointer data);
+static void null_remote_handle (gpointer data);
 
 #define WANT_BITMAPS(x) (x == REPORT_MAIL_USE_ANIMATION || x == REPORT_MAIL_USE_BITMAP)
 
@@ -185,7 +194,8 @@ mailcheck_execute_shell (MailCheck  *mailcheck,
 {
 	GError *error = NULL;
 
-	g_spawn_command_line_async (command, &error);
+	egg_screen_execute_command_line_async (
+		gtk_widget_get_screen (GTK_WIDGET (mailcheck->applet)), command, &error);
 	if (error) {
 		GtkWidget *dialog;
 
@@ -254,17 +264,67 @@ calc_dir_contents (char *dir)
        return size;
 }
 
-static char *
+static void
+check_remote_mailbox (MailCheck *mc)
+{
+	if (!mc->real_password || !mc->remote_username || !mc->remote_server)
+		return;
+
+	if (mc->mailbox_type == MAILBOX_POP3)
+		mc->remote_handle = helper_pop3_check (got_remote_answer,
+						       mc,
+						       null_remote_handle,
+						       mc->pre_remote_command,
+						       mc->remote_server,
+						       mc->remote_username,
+						       mc->real_password);
+	else if (mc->mailbox_type == MAILBOX_IMAP)
+		helper_imap_check (got_remote_answer,
+				   mc,
+				   null_remote_handle,
+				   mc->pre_remote_command,
+				   mc->remote_server,
+				   mc->remote_username,
+				   mc->real_password,
+				   mc->remote_folder);
+}
+
+static void
+password_response_cb (GtkWidget  *dialog,
+		      int         response_id,
+		      MailCheck  *mc)
+{
+
+	switch (response_id) {
+		GtkWidget *entry;
+
+	case GTK_RESPONSE_OK:
+		entry = g_object_get_data (G_OBJECT (dialog), "password_entry");
+		mc->real_password = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+		check_remote_mailbox (mc);
+		break;
+	}
+
+	gtk_widget_destroy (dialog);
+	mc->password_dialog = NULL;
+}
+static void
 get_remote_password (MailCheck *mc)
 {
 	GtkWidget *dialog;
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *entry;
-	int        response;
-	char      *pass = NULL;
 
-	dialog = gtk_dialog_new_with_buttons (
+	if (mc->password_dialog) {
+		gtk_window_set_screen (GTK_WINDOW (mc->password_dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
+		gtk_window_present (GTK_WINDOW (mc->password_dialog));
+		return;
+	}
+
+	mc->password_dialog = dialog =
+		gtk_dialog_new_with_buttons (
 			_("Inbox Monitor"), NULL, 0,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
@@ -291,18 +351,14 @@ get_remote_password (MailCheck *mc)
 	gtk_widget_show_all (hbox);
 	gtk_widget_grab_focus (GTK_WIDGET (entry));
 
-	gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
 
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	g_signal_connect (dialog, "response",
+                          G_CALLBACK (password_response_cb), mc);
 
-	if (response == GTK_RESPONSE_OK)
-		pass = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
-
-	gtk_widget_destroy (dialog);
-
-	return pass;
+	g_object_set_data (G_OBJECT (dialog), "password_entry", entry);
+	gtk_widget_show (GTK_WIDGET (dialog));
 }
 
 static void
@@ -331,6 +387,9 @@ got_remote_answer (int mails, gpointer data)
 						 GTK_BUTTONS_CLOSE,
 						 _("The Inbox Monitor failed to check your mails and thus automatic updating has been deactivated for now.\nMaybe you used a wrong server, username or password?")); 
 
+		gtk_window_set_screen (GTK_WINDOW (dialog),
+				       gtk_widget_get_screen (GTK_WIDGET (mc->applet)));
+
 		g_signal_connect_swapped (G_OBJECT (dialog), "response",
 					  G_CALLBACK (gtk_widget_destroy),
 					  dialog);
@@ -350,6 +409,14 @@ got_remote_answer (int mails, gpointer data)
 	} 
 }
 
+static void
+applet_realized_cb (GtkWidget *widget, gpointer data)
+{
+	MailCheck *mc = data;
+	mail_check_timeout (mc);
+	g_signal_handler_disconnect (G_OBJECT(widget), mc->applet_realized_signal);
+}
+
 static void
 null_remote_handle (gpointer data)
 {
@@ -365,7 +432,6 @@ null_remote_handle (gpointer data)
 static void
 check_mail_file_status (MailCheck *mc)
 {
-	static off_t oldsize = 0;
 	off_t newsize;
 	struct stat s;
 	int status;
@@ -380,45 +446,16 @@ check_mail_file_status (MailCheck *mc)
 		    mc->remote_password[0] != '\0') {
 			g_free (mc->real_password);
 			mc->real_password = g_strdup (mc->remote_password);
-		}
-		else if(mc->real_password == NULL) {
-			if(mc->mail_timeout != 0) {
-				gtk_timeout_remove(mc->mail_timeout);
-				mc->mail_timeout = 0;
-			}
-			mc->real_password = get_remote_password(mc);
-			mc->mail_timeout = gtk_timeout_add(mc->update_freq,
-							   mail_check_timeout,
-							   mc);
-		}
 
-		if (mc->real_password != NULL &&
-		    mc->remote_username != NULL &&
-		    mc->remote_server != NULL) {
-			if (mc->mailbox_type == MAILBOX_POP3)
-				mc->remote_handle =
-					helper_pop3_check (got_remote_answer,
-							   mc,
-							   null_remote_handle,
-							   mc->pre_remote_command,
-							   mc->remote_server,
-							   mc->remote_username,
-							   mc->real_password);
-			else
-					helper_imap_check (got_remote_answer,
-							   mc,
-							   null_remote_handle,
-							   mc->pre_remote_command,
-							   mc->remote_server,
-							   mc->remote_username,
-							   mc->real_password,
-							   mc->remote_folder);
-		}
+		} else if (!mc->real_password)
+			get_remote_password (mc);
+
+		check_remote_mailbox (mc);
 	}
 	else if (mc->mailbox_type == MAILBOX_LOCAL) {
 		status = stat (mc->mail_file, &s);
 		if (status < 0) {
-			oldsize = 0;
+			mc->oldsize = 0;
 			mc->anymail = mc->newmail = mc->unreadmail = 0;
 			after_mail_check (mc);
 			return;
@@ -428,12 +465,12 @@ check_mail_file_status (MailCheck *mc)
 		mc->anymail = newsize > 0;
 		mc->unreadmail = (s.st_mtime >= s.st_atime && newsize > 0);
 		
-		if (newsize != oldsize && mc->unreadmail)
+		if (newsize != mc->oldsize && mc->unreadmail)
 			mc->newmail = 1;
 		else
 			mc->newmail = 0;
 		
-		oldsize = newsize;
+		mc->oldsize = newsize;
 
 		after_mail_check (mc);
 	}
@@ -444,9 +481,9 @@ check_mail_file_status (MailCheck *mc)
 		newmail = calc_dir_contents(tmp);
 		g_snprintf(tmp, sizeof (tmp), "%s/cur", mc->mail_file);
 		oldmail = calc_dir_contents(tmp);
-		mc->newmail = newmail > oldsize;
+		mc->newmail = newmail > mc->oldsize;
 		mc->unreadmail = newmail;
-		oldsize = newmail;
+		mc->oldsize = newmail;
 		mc->anymail = newmail || oldmail;
 		mc->totalmail = newmail + oldmail;
 
@@ -501,10 +538,17 @@ mailcheck_load_animation (MailCheck *mc, const char *fname)
 		mc->report_mail_mode = REPORT_MAIL_USE_ANIMATION;
 	mc->nframe = 0;
 
-	gdk_pixbuf_render_pixmap_and_mask(pb,
-					  &mc->email_pixmap,
-					  &mc->email_mask,
-					  128);
+	mc->email_pixmap = gdk_pixmap_new (gdk_screen_get_root_window (
+					   gtk_widget_get_screen (GTK_WIDGET (mc->applet))),
+					   width, height, -1);
+
+	gdk_pixbuf_render_to_drawable (pb,
+				       mc->email_pixmap,
+				       mc->da->style->black_gc,
+				       0, 0, 0, 0,
+				       width, height,
+				       GDK_RGB_DITHER_NORMAL,
+				       0, 0);
 
 	g_object_unref (pb);
 	
@@ -1579,18 +1623,9 @@ static void
 phelp_cb (GtkDialog *w, gint tab, MailCheck *mc)
 {
 	GError *error = NULL;
-	static GnomeProgram *applet_program = NULL;
-
-	if (!applet_program) {
-		int argc = 1;
-		char *argv[2] = { "mailcheck" };
-		applet_program = gnome_program_init ("mailcheck", VERSION,
-						      LIBGNOME_MODULE, argc, argv,
-     						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-	}
 
 	egg_help_display_desktop_on_screen (
-			applet_program, "mailcheck", "mailcheck", "mailcheck-prefs",
+			NULL, "mailcheck", "mailcheck", "mailcheck-prefs",
 			gtk_widget_get_screen (GTK_WIDGET (mc->applet)),
 			&error);
 	if (error) {
@@ -1743,7 +1778,7 @@ mailcheck_about(BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 	g_free (file);
 	
 	mc->about = gnome_about_new (_("Inbox Monitor"), "1.1",
-				     _("(c) 1998-2000 the Free Software Foundation"),
+				     "Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
 				     _("Inbox Monitor notifies you when new mail arrives in your mailbox"),
 				     authors,
 				     documenters,
@@ -1787,18 +1822,9 @@ static void
 help_callback (BonoboUIComponent *uic, MailCheck *mc, const gchar *verbname)
 {
 	GError *error = NULL;
-	static GnomeProgram *applet_program = NULL;
-
-	if (!applet_program) {
-		int argc = 1;
-		char *argv[2] = { "mailcheck" };
-		applet_program = gnome_program_init ("mailcheck", VERSION,
-						      LIBGNOME_MODULE, argc, argv,
-						      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-	}
 
 	egg_help_display_desktop_on_screen (
-		applet_program, "mailcheck", "mailcheck",NULL,
+		NULL, "mailcheck", "mailcheck",NULL,
 		gtk_widget_get_screen (GTK_WIDGET (mc->applet)),
 		&error);
 	if (error) {
@@ -1886,6 +1912,8 @@ fill_mailcheck_applet(PanelApplet *applet)
 	mc->anymail = mc->unreadmail = mc->newmail = FALSE;
 	mc->mail_timeout = 0;
 	mc->animation_tag = 0;
+	mc->password_dialog = NULL;
+	mc->oldsize = 0;
 
 	/*initial state*/
 	mc->report_mail_mode = REPORT_MAIL_USE_ANIMATION;
@@ -1939,10 +1967,17 @@ fill_mailcheck_applet(PanelApplet *applet)
 	gtk_widget_show_all (GTK_WIDGET (applet));
 
 	/*
-	 * check the mail right now, so we don't have to wait
-	 * for the first timeout
+	 * check the mail if the applet is  realized. Checking the mail 
+	 * right now (in case the applet is not realized), will give us 
+	 * wrong screen value. 
 	 */
-	mail_check_timeout (mc);
+
+	if (GTK_WIDGET_REALIZED (GTK_WIDGET (applet)))
+		mail_check_timeout (mc);
+	else
+		mc->applet_realized_signal =
+			g_signal_connect (G_OBJECT(applet), "realize",
+					  G_CALLBACK(applet_realized_cb), mc);
 
 	return(TRUE);
 }
