@@ -256,8 +256,9 @@ launch_selected (GtkTreeModel *model,
 					terminal->active);
 
 	if (!panel_ditem_launch (
+			ditem, NULL, 0,
 			gtk_window_get_screen (GTK_WINDOW (run_dialog)),
-			ditem, NULL, 0, &error)) {
+			 &error)) {
 		panel_error_dialog (
 			gtk_window_get_screen (GTK_WINDOW (run_dialog)),
 			"failed_to_load_desktop",
@@ -328,7 +329,7 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
                         goto return_and_close;
 
 		escaped = g_markup_escape_text (s, -1);
-		disk = g_filename_from_utf8 (s, -1, NULL, NULL, NULL);
+		disk = g_locale_from_utf8 (s, -1, NULL, NULL, NULL);
 
 		/* save command in history */
 		gnome_entry = g_object_get_data (G_OBJECT (w), "gnome_entry");
@@ -359,17 +360,18 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 			start_geginv ();
                         goto return_and_close;
 		} else if (strcmp (s, "End world hunger") == 0) {
-			egg_screen_url_show (
+			egg_url_show_on_screen (
+				"http://www.wfp.org",
 				gtk_window_get_screen (GTK_WINDOW (run_dialog)),
-				"http://www.wfp.org", NULL);
+				NULL);
                         goto return_and_close;
 		}
 
                 /* Somewhat of a hack I suppose */
                 if (panel_is_url (s)) {
 			/* FIXME: URLs are in UTF8 ... right? */
-                        egg_screen_url_show (
-				gtk_window_get_screen (GTK_WINDOW (run_dialog)), s, NULL);
+                        egg_url_show_on_screen (
+				s, gtk_window_get_screen (GTK_WINDOW (run_dialog)), NULL);
                         goto return_and_close;
                 }
 
@@ -377,13 +379,17 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 		 * even though it could contain strings, but more likely
 		 * it is all filenames and thus should be in disk encoding */
                 if ( ! g_shell_parse_argv (disk, &temp_argc, &temp_argv, &error)) {
-			panel_error_dialog (
-				gtk_window_get_screen (GTK_WINDOW (run_dialog)),
-				"run_error",
-				_("<b>Failed to execute command:</b> '%s'\n\nDetails: %s"),
-				escaped, error->message);
-			g_clear_error (&error);
-                        goto return_and_close;
+                        g_clear_error (&error);
+                        error = NULL;
+                        if ( ! g_shell_parse_argv (s, &temp_argc, &temp_argv, &error)) {
+                                panel_error_dialog (
+                                                    gtk_window_get_screen (GTK_WINDOW (run_dialog)),
+                                                    "run_error",
+                                                    _("<b>Failed to execute command:</b> '%s'\n\nDetails: %s"),
+                                                    escaped, error->message);
+                                g_clear_error (&error);
+                                goto return_and_close;
+                        }
                 }
 
                 get_environment (
@@ -454,7 +460,8 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 			}
 			
 			info = gnome_vfs_file_info_new ();
-			if (gnome_vfs_get_file_info (path, info, 0) != GNOME_VFS_OK) {
+			if (gnome_vfs_get_file_info (path,info,
+						     GNOME_VFS_FILE_INFO_FOLLOW_LINKS) != GNOME_VFS_OK) {
 				panel_error_dialog(
 					gtk_window_get_screen (GTK_WINDOW (run_dialog)),
 					"run_error",
@@ -465,7 +472,9 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 				goto return_and_close;
 			}
 			
-			if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
+			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
+				command = g_strconcat ("nautilus ", path, NULL);
+			} else {
 				char *mime_info;
 				GnomeVFSMimeApplication *app;				
 				
@@ -491,9 +500,6 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 					g_free (path);
 					goto return_and_close;
 				}
-
-			} else if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
-				command = g_strconcat ("nautilus ", path, NULL);
 			}
 			
 			if (!egg_screen_execute_command_line_async (
@@ -579,6 +585,7 @@ browse_ok (GtkWidget *widget, GtkFileSelection *fsel)
 	fname = gtk_file_selection_get_filename (fsel);
 	if (fname != NULL) {
 		append_file (entry, fname);
+		gtk_widget_grab_focus (entry);
 	}
 	
 	gtk_widget_destroy (GTK_WIDGET (fsel));
@@ -1178,9 +1185,11 @@ find_icon_timeout (gpointer data)
 		if (icon != NULL) {
 			pixbuf = gdk_pixbuf_new_from_file (icon, NULL);
 			g_free (icon);
-			if (pixbuf != NULL)
+			if (pixbuf != NULL) {
 				gtk_image_set_from_pixbuf (GTK_IMAGE (pixmap),
 							   pixbuf);
+				g_object_unref (pixbuf);
+			}
 		}
 		g_free (found_icon);
 	}
@@ -1232,6 +1241,7 @@ add_icon_idle (GtkListStore *list)
 			gtk_list_store_set (list, &iter, COLUMN_ICON, pixbuf, -1);
 			g_object_unref (pixbuf);
 		}
+		g_free (file);
 	/* don't go back into the main loop if this wasn't very hard to do */
 	} while (!long_operation);
 
@@ -1473,6 +1483,7 @@ selection_changed (GtkTreeSelection *selection,
                         
                         if (pixbuf != NULL) {
 				gtk_image_set_from_pixbuf (GTK_IMAGE (gpixmap), pixbuf);
+				g_object_unref (pixbuf);
                         } else {
                                 unset_pixmap (gpixmap);
                         }
@@ -1556,46 +1567,59 @@ create_program_list_contents (void)
 static void
 update_contents (GtkWidget *dialog)
 {
-        GtkWidget *program_list_box = NULL;
-        gboolean show_program_list;
+	GtkWidget *program_list_box = NULL;
+	GtkWidget *entry;           
+	gboolean show_program_list;
 	const char *key;
 
 	key = panel_gconf_general_key
 		(panel_gconf_get_profile (), "show_program_list"),
 	show_program_list = panel_gconf_get_bool (key, SHOW_LIST_DEFAULT);
         
-        if (show_program_list) {
-                program_list_box = g_object_get_data (G_OBJECT (dialog), "program_list_box");
+	if (show_program_list) {
+		program_list_box = g_object_get_data (G_OBJECT (dialog),
+						      "program_list_box");
 
-                if (program_list_box && program_list_box->parent == NULL) {
+		if (program_list_box && program_list_box->parent == NULL) {
 			GtkWidget *list;
-			
+			GtkTreeSelection *selection;
+			GtkTreeModel *model;
+			GtkTreeIter iter;
+
 			gtk_window_resize (GTK_WINDOW (dialog), 100, 300);
 			gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
-			
-                        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-                                            program_list_box,
-                                            TRUE, TRUE, GNOME_PAD_SMALL);
-                
 
-                        gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
+			gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+					    program_list_box,
+					    TRUE, TRUE, GNOME_PAD_SMALL);
+
+			gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
 
 			list = g_object_get_data (G_OBJECT (dialog), "program_list");
-			gtk_widget_grab_focus (list);
-		}
-        } else {    
-		GtkWidget *entry;           
+			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
 
-                program_list_box = g_object_get_data (G_OBJECT (dialog), "program_list_box");
+			if (gtk_tree_view_get_model (GTK_TREE_VIEW (list)) &&
+			    !gtk_tree_selection_get_selected (selection,&model, &iter)) {
+			
+				entry = g_object_get_data (G_OBJECT (dialog), "entry");
+				gtk_widget_grab_focus (entry);
+			} else
+				gtk_widget_grab_focus (list);
+		}
+	} else {    
+
+		program_list_box = g_object_get_data (G_OBJECT (dialog),
+						      "program_list_box");
                 
-                if (program_list_box && program_list_box->parent != NULL) {
-                        gtk_container_remove (GTK_CONTAINER (program_list_box->parent), program_list_box);
+		if (program_list_box && program_list_box->parent != NULL) {
+			gtk_container_remove (GTK_CONTAINER (program_list_box->parent),
+					      program_list_box);
 			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 		}
 
 		entry = g_object_get_data (G_OBJECT (dialog), "entry");
-                gtk_widget_grab_focus (entry);
-        }
+		gtk_widget_grab_focus (entry);
+	}
 }
 
 static void
@@ -1688,12 +1712,15 @@ show_run_dialog (GdkScreen *screen)
 		add_items_idle_id =
 			g_idle_add_full (G_PRIORITY_LOW, add_items_idle,
 			 		 w, NULL);
-	}
 
-	/* always focus the entry initially */
+	} 
+
+	/* None of the apps in the list are selected when the dialog is launched.
+	   Hence entry can be safely grabbed
+	*/
 	w = g_object_get_data (G_OBJECT (run_dialog), "entry");
 	gtk_widget_grab_focus (w);
-
+	
 	gtk_widget_show_all (run_dialog);
 }
 
@@ -1702,6 +1729,7 @@ show_run_dialog_with_text (GdkScreen  *screen,
 			   const char *text)
 {
 	GtkWidget *entry;
+	char *exec;
 
 	g_return_if_fail (text != NULL);
 
@@ -1713,5 +1741,9 @@ show_run_dialog_with_text (GdkScreen  *screen,
         
 	entry = g_object_get_data (G_OBJECT (run_dialog), "entry");
 
-	gtk_entry_set_text (GTK_ENTRY (entry), text);
+	exec = remove_parameters (text);
+
+	gtk_entry_set_text (GTK_ENTRY (entry), exec);
+
+	g_free (exec);
 }
